-- wTaskNote.lua: Sep 2010
-- Main LUA file, controls the minimap icon and the options dialog

-- Localization, works in conjunction with the wowace.com website
local L = LibStub("AceLocale-3.0"):GetLocale("wTaskNote")

-- Create a basic AceAddon object to base everything off of
-- See: http://www.wowace.com/addons/ace3/pages/getting-started/
-- We mixin the AceConsole bit so that our addon can have slash commands
wTaskNote = LibStub("AceAddon-3.0"):NewAddon("wTaskNote", "AceConsole-3.0")

-- Use LDBIcon to do the minimap icon (similar to how Omen does it)
-- http://forums.wowace.com/showthread.php?t=15728
local LDB = LibStub("LibDataBroker-1.1", true)
local LDBIcon = LDB and LibStub("LibDBIcon-1.0", true)

-----------------------------------------------------------------------------
-- DB defaults (see AceDB3)
-- Referencing things here is like: self.db.global.minimap.hide because the "db" object is a child of wTaskNote
-- Each scope gets its own "firstuse" variable so that we know whether the scope needs to have samples added
local wTaskNoteDefaults = {
	global = {
		minimap = {
			hide = false,
			minimapPos = 250,
			radius = 80,
		},
		firstuse = true,
	},
	realm = {
		firstuse = true,
	},
	char = {
		firstuse = true,
	},
}

-----------------------------------------------------------------------------
-- DATA DESIGN NOTES (see AceDB3 documentation)
-- self.db.global.* = stuff that gets stored per-account
-- self.db.realm.* = per-realm data
-- self.db.char.* = per-character data
-- self.db.profile.* = per-profile data
-- 
-- Notes get stored one layer below global/realm/char/profile
-- self.db.*.notes.[1..n] = categories (stored as sub-tables)
-- self.db.*.notes.[1..n].[0] = category name string
-- self.db.*.notes.[1..n].[1..n] = { notename = "string", notebody = "string" }
-- 
-- by putting the category name in index position zero, we can still sort the contents
-- of the individual category tables because the table sort function only looks 
-- at indexes [1..n]
--
-- we'll need (2) compare functions, one for sorting the category elements
-- and another for sorting the note table elements within each category table
-- 
-- See: http://www.wowwiki.com/API_ipairs
-- See: http://www.lua.org/manual/5.1/manual.html#5.5
-- See: http://lua-users.org/wiki/TableLibraryTutorial
-- 
-- The following tables get tacked on to the database if we decide that this is
-- a first use (self.db.global.firstuse == true)
local wTaskNoteDefaultNotes = {
	global = {
		notes = {
			[1] = {
				[0] = "Default category",
				[1] = {
					notename = "Example note #1",
					notebody = "Body of example note #1",
				},
				[1] = {
					notename = "Example note #2",
					notebody = "Body of example note #2",
				},
			},
			[2] = {
				[0] = "Second category",
				[1] = {
					notename = "Example note #4",
					notebody = "Body of example note #4",
				},
				[1] = {
					notename = "Example note #3",
					notebody = "Body of example note #3",
				},
			},
			[3] = {
				[0] = "category #3",
				[1] = {
					notename = "Example note #6",
					notebody = "Body of example note #6",
				},
				[1] = {
					notename = "Example note #5",
					notebody = "Body of example note #5",
				},
			},
		},
	},
	realm = {
		notes = {
			[1] = {
				[0] = "Default realm category",
				[1] = {
					notename = "Example realm note #3",
					notebody = "Body of realm example note #3",
				},
				[2] = {
					notename = "Example realm note #2",
					notebody = "Body of realm example note #2",
				},
				[3] = {
					notename = "Example realm note #1",
					notebody = "Body of realm example note #1",
				},
			},
		},
	},
	char = {
		notes = {
			[1] = {
				[0] = "Default character",
				[1] = {
					notename = "Example character note #1",
					notebody = "Body of character example note #1",
				},
				[2] = {
					notename = "Example character note #3",
					notebody = "Body of character example note #3",
				},
				[3] = {
					notename = "Example character note #2",
					notebody = "Body of character example note #2",
				},
			},
		},
	},
}

-----------------------------------------------------------------------------
-- Options table for AceConfig3
-- Works with the Ace3 libraries to setup and control the options menu
-- Right now, we're just doing a checkbox to show/hide the minimap button
local wTaskNoteOptions = {
	name = L["Options Title"],
	handler = wTaskNote,
	type = "group",
	args = {
		MinimapIcon = {
			type = "toggle",
			name = L["Options Show MiniMap Button"],
			desc = L["Options Show MiniMap Button Tooltip"],
			order = 1,
			get = function(info) return not wTaskNote.db.global.minimap.hide end,
			set = function(info, value)
				wTaskNote.db.global.minimap.hide = not value
				if value then LDBIcon:Show("wTNL") else LDBIcon:Hide("wTNL") end
			end,
			hidden = function() return not LDBIcon or IsAddOnLoaded("Broker2FuBar") or IsAddOnLoaded("FuBar") end,
			}
		}
	}

-----------------------------------------------------------------------------
-- OnInitialize(), Called by AceAddon when the addon is first loaded by the 
-- game client.  We'll be restoring saved settings here.  See also:
-- http://www.wowace.com/addons/ace3/pages/api/ace-addon-3-0/
function wTaskNote:OnInitialize()
	self:Print("OnInitialize()")

	-- Initialize our persistent storage database (using AceDB3)
	self.db = LibStub("AceDB-3.0"):New("wTaskNoteDB", wTaskNoteDefaults)
	wTaskNote:DebugWalkTable(self.db, "self.db")
	
	-- if this is our first use, then populate the data table with
	-- a default set of notes to demonstrate functionality
	wTaskNote:InsertDefaultNotesOnFirstUse()
	
	-- Notes should then be sorted before any UI gets displayed
	-- This function should be stable enough that you can call it any time that the list of notes changes
	
	-- Options
	LibStub("AceConfig-3.0"):RegisterOptionsTable("wTaskNote", wTaskNoteOptions, nil)
	self.options = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("wTaskNote", L["wTaskNote"])
	
	-- Add console command of /wtn
	self:RegisterChatCommand("wtn", "ChatCommand")
	
	-- Minimap icon using LibDBIcon/LibDataBroker, similar to how Omen does it
	-- See: http://forums.wowace.com/showthread.php?t=15728
	-- For list of icons, see:
	-- http://www.wowwiki.com/WoWWiki:WoW_Icons/Icon_List/Images
	-- There are 17 different letter types (01-17): Interface\\Icons\\INV_Letter_##.blp
	-- There are 6 different note types (01-06): Interface\\Icons\\INV_Misc_Note_##.blp
	-- Note #3 has "1, 2, 3" on it, note #4 or #5 looks like a possible notes
	-- Also partial to letter #3, #6, #9, #11 or #14
	if LDB then
		wTaskNoteLauncher = LDB:NewDataObject("wTaskNote", { 
			type = "launcher",
			icon = "Interface\\Icons\\INV_Letter_06.blp",
			OnClick = function (frame, button)
				if button == "RightButton" then
					self:ShowOptions()
				else
					self:ToggleNoteFrame()
				end
			end,
			OnTooltipShow = function(tooltip)
				tooltip:AddLine(L["MinimapTooltip Title"])
				tooltip:AddLine(L["MinimapTooltip LeftClick"])
				tooltip:AddLine(L["MinimapTooltip RightClick"])
			end,
		})
		-- Had to change our minimap icon name from "wTaskNote" to "wTNL" to keep MinimapButtonFrame 
		-- from seeing us as an uncollectable button
		if LDBIcon and not IsAddOnLoaded("Broker2FuBar") and not IsAddOnLoaded("FuBar") then
			LDBIcon:Register("wTNL", wTaskNoteLauncher, self.db.global.minimap)
		end
	end
	
end

-----------------------------------------------------------------------------
-- OnEnable(), can be called multiple times per session by AceAddon if the 
-- user enables/disables your addon.  Gets called during the PLAYER_LOGIN event
function wTaskNote:OnEnable()
    
end

-----------------------------------------------------------------------------
-- OnDisable(), can be called multiple times per session by AceAddon if the 
-- user enables/disables your addon.
function wTaskNote:OnDisable()
    
end

-----------------------------------------------------------------------------
-- Shows the options panel
function wTaskNote:ShowOptions()
	InterfaceOptionsFrame_OpenToCategory(self.options)
end

-----------------------------------------------------------------------------
-- 
function wTaskNote:ChatCommand()
end

-----------------------------------------------------------------------------
-- On the first use of a particular scope, insert default notes into the data tables so that users have something to look at.  
-- This also ensures that there is always at least 1 category in each scope
function wTaskNote:InsertDefaultNotesOnFirstUse()
	if (self.db.global.firstuse) then
		self.db.global.notes = wTaskNoteDefaultNotes.global.notes
		self.db.global.firstuse = false
	end		
	if (self.db.realm.firstuse) then
		self.db.realm.notes = wTaskNoteDefaultNotes.realm.notes
		self.db.realm.firstuse = false
	end		
	if (self.db.char.firstuse) then
		self.db.char.notes = wTaskNoteDefaultNotes.char.notes
		self.db.char.firstuse = false
	end		
end

-----------------------------------------------------------------------------
-- walk the linked table, output to the console
-- will call itself is the selected key's value is a table in order to walk a sub-table
function wTaskNote:DebugWalkTable(lua_table, levelpfx)
	--self:Print("DebugWalkDatabase()")
	
	for key,value in pairs(lua_table) do

		-- possible types per the Lua 5.1 manual returned by type() function
		-- "number", "string", "boolean", "table", "function", "thread", and "userdata"
		
		-- some types of key identifiers need to be converted to strings prior to concatenation
		if (type(key) == "number") then
			key = tostring(key)
		elseif (type(key) == "boolean") then
			if (key) then 
				key = "true"
			else
				key = "false"
			end
		end

		keypfx = levelpfx .. "." .. key .. "(" .. type(key) .. "):(" .. type(value) .. ")"
		-- DEBUG: Spit out the key and information about the value's type		
		--self:Print(keypfx)

		-- examine the value types, output information about the key/value pair
		if (type(value) == "number") then
			self:Print(keypfx .. " " .. tostring(value))
		elseif (type(value) == "string") then
			self:Print(keypfx .. " " .. value)
		elseif (type(value) == "boolean") then
			self:Print(keypfx .. " " .. tostring(value))
		elseif (type(value) == "table") then
			self:Print(keypfx .. " " .. "(table)")
			-- call ourselves again to walk the sub-table
			wTaskNote:DebugWalkTable(value, levelpfx .. "." .. key)
		elseif (type(value) == "function") then
			self:Print(keypfx .. " " .. "function()")
		elseif (type(value) == "thread") then
			self:Print(keypfx .. " " .. "thread()")
		elseif (type(value) == "userdata") then
			self:Print(keypfx .. " " .. "<userdata>")
		else
			self:Print(keypfx .. " " .. "?other?")
		end
	end
end
